/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __DOTNET_LAUNCHER_PLUGIN_H__
#define __DOTNET_LAUNCHER_PLUGIN_H__

#include "coreclr_host.h"

extern "C"
{
	/**
	 * @brief initialize plugin code
	 * @param[in] mode mode of plugin ("default", "inhouse", etc)
	 */
	void plugin_initialize(const char* mode);

	/**
	 * @brief preload libraries. this fuction is called in the candidate process only
	 */
	void plugin_preload();

	/**
	 * @brief set appInfo to plugin
	 * @param[in] appID application ID to launch
	 * @param[in] assemblyPath assembly path which has entry point
	 */
	void plugin_set_app_info(const char* appId,
							 const char* assemblyPath);

	/**
	 * @brief set coreclr info to plugin
	 * @param[in] hostHandle host handle of coreclr
	 * @param[in] domainId current domain ID
	 * @param[in] coreclr_create_delegate_ptr function pointer of coreclr_create_delegate
	 *            delegate function can be used to preload managed code
	 */
	void plugin_set_coreclr_info(void* hostHandle,
								 unsigned int domainId,
								 coreclr_create_delegate_ptr delegateFunc);

	/**
	 * @brief return additional pathes to find platform assembly.
	 * @return ":" seperated pathes
	 */
	char* plugin_get_dll_path();

	/**
	 * @brief function will be called before invoking managed code
	 */
	void plugin_before_execute();

	/**
	 * @brief function will be called when application is terminated
	 */
	void plugin_finalize();
}

#endif /* __DOTNET_LAUNCHER_PLUGIN_H__ */
