/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __PLUGIN_MANAGER_H__
#define __PLUGIN_MANAGER_H__

#include "coreclr_host.h"

typedef void (*plugin_initialize_ptr)(const char* mode);
typedef void (*plugin_preload_ptr)();
typedef void (*plugin_set_app_info_ptr)(
			const char* appId,
			const char* managedAssemblyPath);
typedef void (*plugin_set_coreclr_info_ptr)(
			void* hostHandle,
			unsigned int domainId,
			coreclr_create_delegate_ptr delegateFunc);
typedef char* (*plugin_get_dll_path_ptr)();
typedef void (*plugin_before_execute_ptr)();
typedef void (*plugin_finalize_ptr)();

typedef struct PluginFunc {
	plugin_initialize_ptr initialize;
	plugin_preload_ptr preload;
	plugin_set_app_info_ptr set_app_info;
	plugin_set_coreclr_info_ptr set_coreclr_info;
	plugin_get_dll_path_ptr get_dll_path;
	plugin_before_execute_ptr before_execute;
	plugin_finalize_ptr finalize;
} PluginFunc;

// plugin functions
void pluginPreload();
void pluginSetAppInfo(const char* appId, const char* managedAssemblyPath);
void pluginSetCoreclrInfo(void* hostHandle, unsigned int domainId, coreclr_create_delegate_ptr delegateFunc);
char* pluginGetDllPath();
void pluginBeforeExecute();

// initialize / finalize plugin manager
int initializePluginManager(const char* mode);
void finalizePluginManager();

#endif /* __PLUGIN_MANAGER_H__ */
